/*
** Module   : AbstractClient.java
** Abstract : OS-level window construct for Swing-based clients
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 MAG 20140508 First version.
** 002 GES 20150416 Moved to a Swing-specific package.
*/

package com.goldencode.p2j.ui.client.driver.swing;

import javax.swing.*;

/**
 * OS-level window construct for Swing-based clients.
 */
@SuppressWarnings("serial")
public abstract class AbstractClient 
extends JFrame 
implements ContainerOps
{
   /**
    * Constructor.
    * 
    * @param    title
    *           Main window title. 
    */
   public AbstractClient(String title)
   {
      super(title);
   }

   /**
    * Initialize.
    * 
    * @param    pane
    *           Content pane.
    * @param    resizable
    *           Resizable attribute.
    */
   protected void init(JComponent pane, boolean resizable)
   {
      setContentPane(pane);
      forceResize();

      // frame-specific init
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      setResizable(resizable);      
   }

   /**
    * Trigger a resizing operation.
    */                                       
   public void forceResize()
   {
      pack();
      setVisible(true);
   }

   /**
    * Report if it is possible to exit the VM using <code>System.exit()</code>.
    * 
    * @return   Always <code>true</code>.
    */
   public boolean canExit()
   {
      return true;
   }
}
