/*
** Module   : p2j.clipboard_helpers.js
** Abstract : ChUI-specific text clipboard processing
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 GES 20150324 First version, with refactored content moved from tty.clipboard.js.
*/

p2j.clipboard_helpers = (function() 
{
   /** Clipboard helper object which will be returned from this initialization function. */
   var me = {};
   
   /** Draw rectangle selection using stroke. */
   var stroke = true;
   
   /** Selection color. */
   var color = 'blue';

   /** Font width metric. */
   var charWidth;
   
   /** Font height metric. */
   var charHeight;
      
   /** Canvas object. */
   var canvas;
   
   /** 2D graphics context. */
   var ctx;
   
   /** Selected cells. */
   var cells = {};
   
   /** Selection rectangle. */
   var rect = {};
   
   /**
    * Init module.
    * 
    * @param {type} cfg configurartion object
    */
   me.init = function(cfg)
   {
      charWidth = p2j.screen.charWidth;
      charHeight = p2j.screen.charHeight;

      canvas = document.getElementById(cfg.clipboard.id);
      canvas.width = p2j.screen.canvasWidth;
      canvas.height = p2j.screen.canvasHeight;
      
      stroke = cfg.clipboard.stroke;
      
      ctx = canvas.getContext('2d');
      ctx.fillStyle = "white"; 
      ctx.globalAlpha = 0;
      ctx.clearRect(0, 0, canvas.width, canvas.height);
   };
   
   /**
    * Mouse down event handler.
    * 
    * @param {type} e Event object
    * @returns {undefined}
    */
   me.mouseDown = function(e) 
   {
      var x = (e.pageX - this.offsetLeft) | 0;
      var y = (e.pageY - this.offsetTop) | 0;
      cells.startX = (x / charWidth) | 0;
      cells.startY = (y / charHeight) | 0;         
      rect.startX = cells.startX * charWidth;
      rect.startY = cells.startY * charHeight;
   }

   /**
    * Mouse move event handler.
    * 
    * @param {type} e Event object
    * @returns {undefined}
    */
   me.mouseMove = function(e) 
   {
      var x = (e.pageX - this.offsetLeft | 0);
      var y = (e.pageY - this.offsetTop | 0);
      cells.startX1 = (x / charWidth) | 0;
      cells.startY1 = (y / charHeight) | 0;

      rect.w = (cells.startX1 - cells.startX + 1) * charWidth;
      rect.h = (cells.startY1 - cells.startY + 1) * charHeight;

      ctx.clearRect(0, 0, canvas.width, canvas.height);         

      if (stroke)
      {
         ctx.globalAlpha = 1.0;
         ctx.beginPath();
         ctx.strokeStyle = color;      
         ctx.strokeRect(rect.startX, rect.startY, rect.w, rect.h);
         ctx.stroke();            
      }
      else
      {
         ctx.globalAlpha = 0.2;
         ctx.fillStyle = color;
         ctx.fillRect(rect.startX, rect.startY, rect.w, rect.h);
      }
   }
   
   /**
    * Return the currently selected text.
    * 
    * @return   The text that is selected.
    */
   me.getSelectedText = function()
   {
      // rows
      var begRow = Math.min(cells.startY, cells.startY1);
      var endRow = Math.max(cells.startY, cells.startY1);
      
      // cols
      var begCol = Math.min(cells.startX, cells.startX1);
      var endCol = Math.max(cells.startX, cells.startX1);
      
      return p2j.screen.copy(begRow, endRow, begCol, endCol);
   };
   
   /**
    * Clear selection on screen.
    * 
    * @returns {undefined}
    */
   me.clearSelection = function()
   {
      ctx.clearRect(0, 0, canvas.width, canvas.height);
   };
   
   /**
    * Add the given event listener to the canvas.
    * 
    * @param    event {string}
    *           The event name for which to register.
    * @param    callback {function}
    *           The function to call when that event fires.
    */
   me.addEventListener = function(event, callback)
   {
      canvas.addEventListener(event, callback, false);
   };

   return me;   
}());
